#pragma once 

#ifndef EASING_TYPE
  #define EASING_TYPE float
#endif

namespace easing
{
  // Linear interpolation (no easing)
  EASING_TYPE LinearInterpolation(EASING_TYPE p);

  // Quadratic easing; p^2
  EASING_TYPE QuadraticEaseIn(EASING_TYPE p);
  EASING_TYPE QuadraticEaseOut(EASING_TYPE p);
  EASING_TYPE QuadraticEaseInOut(EASING_TYPE p);

  // Cubic easing; p^3
  EASING_TYPE CubicEaseIn(EASING_TYPE p);
  EASING_TYPE CubicEaseOut(EASING_TYPE p);
  EASING_TYPE CubicEaseInOut(EASING_TYPE p);

  // Quartic easing; p^4
  EASING_TYPE QuarticEaseIn(EASING_TYPE p);
  EASING_TYPE QuarticEaseOut(EASING_TYPE p);
  EASING_TYPE QuarticEaseInOut(EASING_TYPE p);

  // Quintic easing; p^5
  EASING_TYPE QuinticEaseIn(EASING_TYPE p);
  EASING_TYPE QuinticEaseOut(EASING_TYPE p);
  EASING_TYPE QuinticEaseInOut(EASING_TYPE p);

  // Sine wave easing; sin(p * PI/2)
  EASING_TYPE SineEaseIn(EASING_TYPE p);
  EASING_TYPE SineEaseOut(EASING_TYPE p);
  EASING_TYPE SineEaseInOut(EASING_TYPE p);

  // Circular easing; sqrt(1 - p^2)
  EASING_TYPE CircularEaseIn(EASING_TYPE p);
  EASING_TYPE CircularEaseOut(EASING_TYPE p);
  EASING_TYPE CircularEaseInOut(EASING_TYPE p);

  // Exponential easing, base 2
  EASING_TYPE ExponentialEaseIn(EASING_TYPE p);
  EASING_TYPE ExponentialEaseOut(EASING_TYPE p);
  EASING_TYPE ExponentialEaseInOut(EASING_TYPE p);

  // Exponentially-damped sine wave easing
  EASING_TYPE ElasticEaseIn(EASING_TYPE p);
  EASING_TYPE ElasticEaseOut(EASING_TYPE p);
  EASING_TYPE ElasticEaseInOut(EASING_TYPE p);

  // Overshooting cubic easing; 
  EASING_TYPE BackEaseIn(EASING_TYPE p);
  EASING_TYPE BackEaseOut(EASING_TYPE p);
  EASING_TYPE BackEaseInOut(EASING_TYPE p);

  // Exponentially-decaying bounce easing
  EASING_TYPE BounceEaseIn(EASING_TYPE p);
  EASING_TYPE BounceEaseOut(EASING_TYPE p);
  EASING_TYPE BounceEaseInOut(EASING_TYPE p);

  EASING_TYPE BounceTwice(EASING_TYPE p);
}